@@ -111,6 +111,8 @@ sub show($c) {
 # List resources from table stranici.
 ## no critic qw(Subroutines::ProhibitBuiltinHomonyms)
 sub index($c) {
+  my $domain = $c->host_only;
+  my $str    = $c->stranici;
   if ($c->current_route =~ /^api\./) {    #invoked via OpenAPI
     $c->openapi->valid_input or return;
     my $input = $c->validation->output;
@@ -118,10 +120,11 @@ sub index($c) {
     # TODO: Modify $input: add where clause, get also title in the requested
     # language from celini and merge it into the stranici object. Modify the
     # Swagger description of response object to conform to the output.
-    return $c->render(openapi => $c->stranici->all($input));
+    return $c->render(openapi => $str->all($input));
   }
-  my $str = $c->stranici;
-  my $opts = {where => $str->readable_by($c->user)};
+  my $opts
+    = {where => {%{$str->readable_by($c->user)}, $str->where_domain_is($domain)}
+    };
 
   return $c->render(stranici => $str->all($opts));
 }
