@@ -1,15 +1,22 @@
 package Slovo::Controller::Upravlenie;
-use Mojo::Base 'Slovo::Controller';
+use Mojo::Base 'Slovo::Controller', -signatures;
 use feature qw(lexical_subs unicode_strings);
 ## no critic qw(TestingAndDebugging::ProhibitNoWarnings)
 no warnings "experimental::lexical_subs";
 
 # This action will render a template
 ## no critic qw(Subroutines::ProhibitBuiltinHomonyms)
-sub index {
-  my $self = shift;
+sub index($c) {
   state $menu = [qw(groups users domove stranici celini)];
-  return $self->render(menu => $menu);
+  return $c->render(
+    menu => [
+      map {
+        $_ eq 'groups' && !$c->groups->is_admin($c->user->{id})
+          ? ()
+          : $_
+        } @$menu
+    ]
+  );
 }
 
 1;
